@@ -575,6 +575,8 @@ def gerrit(registry, xml_parent, data):
         data.get('dynamic-trigger-enabled', False))
     XML.SubElement(gtrig, 'triggerConfigURL').text = str(
         data.get('dynamic-trigger-url', ''))
+    XML.SubElement(gtrig, 'triggerInformationAction').text = str(
+        data.get('trigger-information-action', ''))
     XML.SubElement(gtrig, 'allowTriggeringUnreviewedPatches').text = str(
         data.get('trigger-for-unreviewed-patches', False)).lower()
     build_gerrit_triggers(gtrig, data)
@@ -1928,7 +1930,7 @@ def stash_pull_request(registry, xml_parent, data):
     """yaml: stash-pull-request
     Trigger builds via Stash/Bitbucket Server Pull Requests.
     Requires the Jenkins :jenkins-wiki:`Stash Pull Request Builder Plugin
-      <Stash+pullrequest+builder+plugin>`.
+    <Stash+pullrequest+builder+plugin>`.
 
       :arg str cron: cron syntax of when to run (required)
       :arg str stash-host: The HTTP or HTTPS URL of the Stash host (NOT ssh).
