@@ -29,7 +29,6 @@ import xml.etree.ElementTree as XML
 
 from jenkins_jobs.errors import InvalidAttributeError
 from jenkins_jobs.errors import JenkinsJobsException
-from jenkins_jobs.errors import MissingAttributeError
 import jenkins_jobs.modules.base
 from jenkins_jobs.modules.builders import create_builders
 from jenkins_jobs.modules.helpers import artifactory_common_details
@@ -312,14 +311,19 @@ def timeout(registry, xml_parent, data):
     prefix = 'hudson.plugins.build__timeout.'
     twrapper = XML.SubElement(xml_parent, prefix + 'BuildTimeoutWrapper')
 
-    plugin_info = registry.get_plugin_info(
-        "Jenkins build timeout plugin")
-    version = pkg_resources.parse_version(plugin_info.get("version", "0"))
+    plugin_info = registry.get_plugin_info("Build Timeout")
+    if "version" not in plugin_info:
+        plugin_info = registry.get_plugin_info("Jenkins build timeout plugin")
+    version = plugin_info.get("version", None)
+    if version:
+        version = pkg_resources.parse_version(version)
 
     valid_strategies = ['absolute', 'no-activity', 'likely-stuck', 'elastic',
                         'deadline']
 
-    if version >= pkg_resources.parse_version("1.14"):
+    # NOTE(toabctl): if we don't know the version assume that we
+    # use a newer version of the plugin
+    if not version or version >= pkg_resources.parse_version("1.14"):
         strategy = data.get('type', 'absolute')
         if strategy not in valid_strategies:
             InvalidAttributeError('type', strategy, valid_strategies)
@@ -1728,10 +1732,6 @@ def credentials_binding(registry, xml_parent, data):
             'com.cloudbees.jenkins.plugins.awscredentials'
             '.AmazonWebServicesCredentialsBinding'
     }
-    if not data:
-        raise JenkinsJobsException('At least one binding-type must be '
-                                   'specified for the credentials-binding '
-                                   'element')
     for binding in data:
         for binding_type, params in binding.items():
             if binding_type not in binding_types.keys():
@@ -1741,26 +1741,24 @@ def credentials_binding(registry, xml_parent, data):
             binding_xml = XML.SubElement(bindings_xml,
                                          binding_types[binding_type])
             if binding_type == 'username-password-separated':
-                try:
-                    XML.SubElement(binding_xml, 'usernameVariable'
-                                   ).text = params['username']
-                    XML.SubElement(binding_xml, 'passwordVariable'
-                                   ).text = params['password']
-                except KeyError as e:
-                    raise MissingAttributeError(e.args[0])
+                mapping = [
+                    ('username', 'usernameVariable', None),
+                    ('password', 'passwordVariable', None)]
+                convert_mapping_to_xml(
+                    binding_xml, params, mapping, fail_required=True)
             elif binding_type == 'amazon-web-services':
-                try:
-                    XML.SubElement(binding_xml, 'accessKeyVariable'
-                                   ).text = params['access-key']
-                    XML.SubElement(binding_xml, 'secretKeyVariable'
-                                   ).text = params['secret-key']
-                except KeyError as e:
-                    raise MissingAttributeError(e.args[0])
+                mapping = [
+                    ('access-key', 'accessKeyVariable', None),
+                    ('secret-key', 'secretKeyVariable', None)]
+                convert_mapping_to_xml(
+                    binding_xml, params, mapping, fail_required=True)
             else:
-                variable_xml = XML.SubElement(binding_xml, 'variable')
-                variable_xml.text = params.get('variable')
-            credential_xml = XML.SubElement(binding_xml, 'credentialsId')
-            credential_xml.text = params.get('credential-id')
+                mapping = [('variable', 'variable', None)]
+                convert_mapping_to_xml(
+                    binding_xml, params, mapping, fail_required=False)
+            mapping = [('credential-id', 'credentialsId', None)]
+            convert_mapping_to_xml(binding_xml,
+                params, mapping, fail_required=False)
 
 
 def custom_tools(registry, xml_parent, data):
@@ -2395,6 +2393,86 @@ def version_number(parser, xml_parent, data):
     convert_mapping_to_xml(version_number, data, mapping, fail_required=True)
 
 
+def github_pull_request(parser, xml_parent, data):
+    """yaml: github-pull-request
+    Set GitHub commit status with custom context and message.
+    Requires the Jenkins :jenkins-wiki:`GitHub Pull Request Builder Plugin
+    <GitHub+pull+request+builder+plugin>`.
+
+    :arg bool show-matrix-status: Only post commit status of parent matrix job
+        (default false)
+    :arg str status-context: The context to include on PR status comments
+        (default '')
+    :arg str triggered-status: The status message to set when the build has
+        been triggered (default '')
+    :arg str started-status: The status message to set when the build has
+        been started (default '')
+    :arg str status-url: The status URL to set (default '')
+    :arg bool status-add-test-results: Add test result one-liner to status
+        message (default false)
+    :arg list statuses: List of custom statuses on the commit for when a build
+        is completed
+
+        :Status:
+            * **message** (`str`) -- The message that is appended to a comment
+              when a build finishes with the desired build status. If no status
+              updates should be made when a build finishes with the indicated
+              build status, use "--none--" to alert the trigger. (required)
+            * **result** (`str`) -- Build result. Can be one of 'SUCCESS',
+              'ERROR' or 'FAILURE'. (required)
+
+    Minimal Example:
+
+    .. literalinclude::
+        /../../tests/wrappers/fixtures/github-pull-request-minimal.yaml
+       :language: yaml
+
+    Full Example:
+
+    .. literalinclude::
+        /../../tests/wrappers/fixtures/github-pull-request-full.yaml
+       :language: yaml
+
+    """
+    ghprb = XML.SubElement(
+        xml_parent, 'org.jenkinsci.plugins.ghprb.upstream.GhprbUpstreamStatus'
+    )
+
+    mapping = [
+        # option, xml name, default value
+        ("show-matrix-status", 'showMatrixStatus', False),
+        ("status-context", 'commitStatusContext', ''),
+        ("triggered-status", 'triggeredStatus', ''),
+        ("started-status", 'startedStatus', ''),
+        ("status-url", 'statusUrl', ''),
+        ("status-add-test-results", 'addTestResults', False)
+    ]
+    convert_mapping_to_xml(ghprb, data, mapping, fail_required=True)
+
+    statuses = data.get('statuses', [])
+    if statuses:
+        status_mapping = [
+            ('message', 'message', None),
+            ('result', 'result', ''),
+        ]
+        result_list = ['ERROR', 'SUCCESS', 'FAILURE']
+
+        completed_tag = XML.SubElement(ghprb, 'completedStatus')
+        for status in statuses:
+            result = status.get('result', '')
+            if result not in result_list:
+                raise JenkinsJobsException(
+                    "'result' must be one of: " + ', '.join(result_list))
+
+            result_tag = XML.SubElement(
+                completed_tag,
+                'org.jenkinsci.plugins.ghprb.extensions'
+                '.comments.GhprbBuildResultMessage'
+            )
+            convert_mapping_to_xml(
+                result_tag, status, status_mapping, fail_required=True)
+
+
 class Wrappers(jenkins_jobs.modules.base.Base):
     sequence = 80
 
